-- name: [CS] Kirby
-- description: The Pink Puffball!\n\n\\#ff7777\\This Pack requires Character Select\nto use as a Library!

--[[
    API Documentation for Character Select can be found below:
    https://github.com/Squishy6094/character-select-coop/wiki/API-Documentation

    Use this if you're curious on how anything here works >v<
	(This is an edited version of the Template File by Squishy)
]]

local KIRBY_JUMPS = 3

local TEXT_MOD_NAME = "Kirby"

-- Stops mod from loading if Character Select isn't on
if not _G.charSelectExists then
    djui_popup_create("\\#ffffdc\\\n"..TEXT_MOD_NAME.."\nRequires the Character Select Mod\nto use as a Library!\n\nPlease turn on the Character Select Mod\nand Restart the Room!", 6)
    return 0
end

local E_MODEL_CUSTOM_MODEL = smlua_model_util_get_id("star_metal_geo") -- Located in "actors"

local TEX_KIRB_ICON = get_texture_info("even_fatter") -- Located in "textures"


-- All Located in "sound" Name them whatever you want. Remember to include the .ogg extension
local VOICETABLE_KIRB = {
    --[CHAR_SOUND_OKEY_DOKEY] = 'haii.ogg', -- Starting game
	[CHAR_SOUND_LETS_A_GO] = 'haii.ogg', -- Starting level
	[CHAR_SOUND_PUNCH_YAH] = 'houh.ogg', -- Punch 1
	[CHAR_SOUND_PUNCH_WAH] = 'huh.ogg', -- Punch 2
	[CHAR_SOUND_PUNCH_HOO] = 'hyuu.ogg', -- Punch 3
	[CHAR_SOUND_YAH_WAH_HOO] = {'Silent.ogg', 'Silent.ogg', 'Silent.ogg'}, -- First/Second jump sounds
	[CHAR_SOUND_HOOHOO] = 'haii.ogg', -- Third jump sound
	[CHAR_SOUND_YAHOO_WAHA_YIPPEE] = {'Silent.ogg', 'Silent.ogg'}, -- Triple jump sounds
	[CHAR_SOUND_UH] = 'scream.ogg', -- Wall bonk
	[CHAR_SOUND_UH2] = 'hyuu.ogg', -- Landing after long jump
	[CHAR_SOUND_UH2_2] = 'hyuu.ogg', -- Same sound as UH2; jumping onto ledge
	[CHAR_SOUND_HAHA] = 'yayay.ogg', -- Landing triple jump
	[CHAR_SOUND_YAHOO] = 'jump.ogg', -- Long jump
	[CHAR_SOUND_DOH] = 'houh.ogg', -- Long jump wall bonk
	[CHAR_SOUND_WHOA] = 'ehyuu.ogg', -- Grabbing ledge
	[CHAR_SOUND_EEUH] = 'Silent.ogg', -- Climbing over ledge
	[CHAR_SOUND_WAAAOOOW] = 'scream.ogg', -- Falling a long distance
	[CHAR_SOUND_TWIRL_BOUNCE] = 'tyoy.ogg', -- Bouncing off of a flower spring
	[CHAR_SOUND_GROUND_POUND_WAH] = 'Silent.ogg', 
	[CHAR_SOUND_HRMM] = 'ouhhou.ogg', -- Lifting something
	[CHAR_SOUND_HERE_WE_GO] = 'yayay.ogg', -- Star get
	[CHAR_SOUND_SO_LONGA_BOWSER] = 'yayay.ogg', -- Throwing Bowser
--DAMAGE
	[CHAR_SOUND_ATTACKED] = 'houh.ogg', -- Damaged
	[CHAR_SOUND_PANTING] = 'Silent.ogg', -- Low health
	[CHAR_SOUND_ON_FIRE] = 'scream.ogg', -- Burned
--SLEEP SOUNDS
	[CHAR_SOUND_IMA_TIRED] = 'Silent.ogg', -- Mario feeling tired
	[CHAR_SOUND_YAWNING] = 'Silent.ogg', -- Mario yawning before he sits down to sleep
	[CHAR_SOUND_SNORING1] = 'snore.ogg', -- Snore Inhale
	[CHAR_SOUND_SNORING2] = 'snore2.ogg', -- Exhale
	[CHAR_SOUND_SNORING3] = 'Silent.ogg', -- Sleep talking / mumbling
--COUGHING (USED IN THE GAS MAZE)
	[CHAR_SOUND_COUGHING1] = 'Silent.ogg', -- Cough take 1
	[CHAR_SOUND_COUGHING2] = 'Silent.ogg', -- Cough take 2
	[CHAR_SOUND_COUGHING3] = 'Silent.ogg', -- Cough take 3
--DEATH
	[CHAR_SOUND_DYING] = 'scream.ogg', -- Dying from damage
	[CHAR_SOUND_DROWNING] = 'Silent.ogg', -- Running out of air underwater
	[CHAR_SOUND_MAMA_MIA] = 'scream.ogg' -- Booted out of level
}

-- All Located in "actors"
local CAPTABLE_CHAR = {
    normal = smlua_model_util_get_id("FILENAME_geo"),
    wing = smlua_model_util_get_id("FILENAME_geo"),
    metal = smlua_model_util_get_id("FILENAME_geo"),
}

local PALETTE_CHAR = {
    [PANTS]  = "fd8fc1",
    [SHIRT]  = "fd004c",
    [GLOVES] = "ffffff",
    [SHOES]  = "fd004c",
    [HAIR]   = "fd004c",
    [SKIN]   = "fd8fc1",
    [CAP]    = "fd8fc1",
	[EMBLEM] = "fd004c"
}

local HM_CHARNAME2= {
    label = {
        left = get_texture_info("HealthLeft"),
        right = get_texture_info("HealthRight"),
    },
    pie = {
        [1] = get_texture_info("Pie1"),
        [2] = get_texture_info("Pie2"),
        [3] = get_texture_info("Pie3"),
        [4] = get_texture_info("Pie4"),
        [5] = get_texture_info("Pie5"),
        [6] = get_texture_info("Pie6"),
        [7] = get_texture_info("Pie7"),
        [8] = get_texture_info("Pie8"),
    }
}

local CSloaded = false
local function on_character_select_load()
    CT_CHARNAME2 = _G.charSelect.character_add("Kirby", {"The Super Star Warrior with a moveset!"}, "Chrrli", {r = 231, g = 145, b = 190}, E_MODEL_CUSTOM_MODEL, CT_CHARNAME2, TEX_KIRB_ICON)
    _G.charSelect.character_add_caps(E_MODEL_CUSTOM_MODEL, CAPTABLE_CHAR)
    _G.charSelect.character_add_voice(E_MODEL_CUSTOM_MODEL, VOICETABLE_KIRB)
    _G.charSelect.character_add_celebration_star(E_MODEL_CUSTOM_MODEL, E_MODEL_CUSTOM_STAR, TEX_CUSTOM_STAR_ICON)
    _G.charSelect.character_add_palette_preset(E_MODEL_CUSTOM_MODEL, PALETTE_CHAR)
    _G.charSelect.character_add_health_meter(CT_CHARNAME2, HM_CHARNAME2)

    CSloaded = true
end

local function on_character_sound(m, sound)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_KIRB then return _G.charSelect.voice.sound(m, sound) end
end

local function on_character_snore(m)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_KIRB then return _G.charSelect.voice.snore(m) end
end

local function mario_update(m)
    if not _G.charSelectExists then return end
    if _G.charSelect.character_get_life_icon(0) ~= TEX_KIRB_ICON then return end
	
	if m.action == ACT_JUMP and m.controller.buttonPressed & A_BUTTON ~= 0 then
		set_mario_action(m, ACT_JUMP, 0)
		m.vel.y = 45
		spawn_mist_particles_variable(0.1, 0, 6)
		m.faceAngle.y = m.intendedYaw
	end

	if m.action == ACT_JUMP and m.controller.buttonPressed & A_BUTTON ~= 0 then
		play_character_sound(m, CHAR_SOUND_YAHOO)
	end

	if m.action == ACT_GROUND_POUND_LAND and m.controller.buttonPressed & B_BUTTON ~= 0 then
		m.faceAngle.y = m.intendedYaw
		m.forwardVel = (m.forwardVel * 0.75)
		m.forwardVel = m.forwardVel + 60
		set_mario_action(m, ACT_SLIDE_KICK, 0)
	end
end

hook_event(HOOK_MARIO_UPDATE, mario_update)
hook_event(HOOK_ON_MODS_LOADED, on_character_select_load)
hook_event(HOOK_CHARACTER_SOUND, on_character_sound)
hook_event(HOOK_MARIO_UPDATE, on_character_snore)